@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool::Exif;
 use Image::ExifTool::PLIST;
 
-$VERSION = '1.09';
+$VERSION = '1.10';
 
 sub ConvertPLIST($$);
 
@@ -110,9 +110,13 @@ sub ConvertPLIST($$);
     },
     # 0x0010 - int32s: 1 (SphereStatus, ref 2)
     0x0011 => { # (if defined, there is a live photo associated with the video, #forum13565) (AssetIdentifier, ref 2)
-        Name => 'MediaGroupUUID', #NealKrawetz private communication
-        # (changed in 12.19 from Name => 'ContentIdentifier', #forum8750)
+        Name => 'ContentIdentifier',
+        Notes => 'called MediaGroupUUID when it appears as an XAttr',
+        # - originally called ContentIdentifier, forum8750
+        # - changed in 12.19 to MediaGroupUUID, NealKrawetz private communication
+        # - changed back to ContentIdentifier since Apple writes this to Keys content.identifier (forum14874)
         Writable => 'string',
+        
     },
     # 0x0012 - (QRMOutputType, ref 2)
     # 0x0013 - (SphereExternalForceOffset, ref 2)
@@ -127,7 +131,7 @@ sub ConvertPLIST($$);
         Writable => 'string',
     },
     # 0x0016 - string[29]: "AXZ6pMTOh2L+acSh4Kg630XCScoO\0" (PhotosOriginatingSignature, ref 2)
-    0x0017 => { #forum13565 (only valid if MediaGroupUUID exists) (StillImageCaptureFlags, ref 2)
+    0x0017 => { #forum13565 (only valid if MediaGroupUUID/ContentIdentifier exists) (StillImageCaptureFlags, ref 2)
         Name => 'LivePhotoVideoIndex',
         Notes => 'divide by RunTimeScale to get time in seconds',
     },
@@ -153,7 +157,7 @@ sub ConvertPLIST($$);
     # 0x001e - (OriginatingAppID, ref 2)
     # 0x001f - int32s: 0,1 (PhotosAppFeatureFlags, ref 2)
     0x0020 => { # (ImageCaptureRequestIdentifier, ref 2)
-        Name => 'ImageCaptureReqestID',
+        Name => 'ImageCaptureRequestID',
         Writable => 'string',
         Unknown => 1,
     },
